@@ -4,9 +4,12 @@ import argparse
 import ast
 import csv
 import difflib
+import gzip
 import re
+import sys
 from io import TextIOWrapper
 from typing import Any
+from typing import Callable
 from typing import cast
 from typing import NamedTuple
 
@@ -15,7 +18,7 @@ from matplotlib.axes import Axes
 from matplotlib.figure import Figure
 
 
-__version__ = "0.4.0"
+__version__ = "0.5.0"
 
 
 COL_DTYPE = {"int": int, "float": float, "str": str}
@@ -30,6 +33,8 @@ class Defaults:
     PLOT_Y = "1"
     CROP_COL = "0"
     OUT_FILE = "-"
+    PICK_COLS = None
+    OMIT_COLS = None
 
 
 def main() -> int:
@@ -50,7 +55,7 @@ def create_parser(
 
     _parser.add_argument(
         "in_file",
-        type=argparse.FileType("r"),
+        type=_filetype("r"),
         default=Defaults.IN_FILE,
         help="Input data file. (default: %(default)s)",
     )
@@ -71,49 +76,76 @@ def create_parser(
         help="The delimiter used in the input data file. (default: %(default)r)",
     )
     _parser.add_argument(
+        "-o",
+        "--out-file",
+        type=_filetype("w"),
+        default=Defaults.OUT_FILE,
+        help="File to write the results to (either an image or TSV file. If "
+        "not provided, a matplotlib plot is shown when plotting, and data is "
+        "written to stdout when cropping. (default: %(default)s)",
+    )
+
+    # plot options
+    plot_group = _parser.add_argument_group("plot options")
+    plot_group.add_argument(
         "-x",
         "--plot-x",
         default=Defaults.PLOT_X,
         help="Column to use on the x-axis. A 0-indexed integer or a column "
         "heading string. (default: %(default)s)",
     )
-    _parser.add_argument(
+    plot_group.add_argument(
         "-y",
         "--plot-y",
         default=Defaults.PLOT_Y,
         help="Column to use on the y-axis. A 0-indexed integer or a column "
         "heading string. (default: %(default)s)",
     )
-    _parser.add_argument(
-        "-c",
-        "--crop-col",
-        default=Defaults.CROP_COL,
-        help="Column to search in for --crop-start/--crop-end values. A "
-        "0-indexed integer or a column heading string. (default: %(default)s)",
-    )
-    _parser.add_argument(
+
+    # crop options
+    crop_group = _parser.add_argument_group("crop options")
+    crop_group.add_argument(
         "-s",
-        "--crop-start",
+        "--start",
         help="Start value to use for cropping (inclusive). If the column is "
         "numeric then the row with column entry closest to the given value is used.",
     )
-    _parser.add_argument(
+    crop_group.add_argument(
         "-e",
-        "--crop-end",
+        "--end",
         help="End value to use for cropping (exclusive). If the column is "
         "numeric then the row with column entry closest to the given value "
         "is used. To specify a numeric value relative to --crop-start use "
         "the syntax ':<number>'. Omit this flag to crop from --crop-start to "
         "the end of the file.",
     )
-    _parser.add_argument(
-        "-o",
-        "--out-file",
-        type=argparse.FileType("w"),
-        default=Defaults.OUT_FILE,
-        help="File to write the results to (either an image or TSV file. If "
-        "not provided a matplotlib plot is shown when plotting, and data is "
-        "written to stdout when cropping. (default: %(default)s)",
+    crop_group.add_argument(
+        "-c",
+        "--col",
+        default=Defaults.CROP_COL,
+        help="Column to search in for --crop-start/--crop-end values. A "
+        "0-indexed integer or a column heading string. (default: %(default)s)",
+    )
+    pick_omit_group = crop_group.add_mutually_exclusive_group()
+    pick_omit_group.add_argument(
+        "-p",
+        "--pick",
+        type=_parse_col_list,
+        default=Defaults.PICK_COLS,
+        help="Comma separated list of columns to keep. If specified, only "
+        "columns in this list will appear in the generated output. Columns "
+        "can either be column heading strings or 0-indexed integers "
+        "(intermixing is allowed). By default all columns are retained.",
+    )
+    pick_omit_group.add_argument(
+        "-r",
+        "--omit",
+        type=_parse_col_list,
+        default=Defaults.OMIT_COLS,
+        help="Comma separated list of columns to omit. If specified, columns "
+        "in this list will be removed from the generated output. Columns can "
+        "either be column heading strings or 0-indexed integers (intermixing "
+        "is allowed).",
     )
 
     _parser.set_defaults(handler=handler)
@@ -121,11 +153,37 @@ def create_parser(
     return _parser
 
 
+def _filetype(mode: str) -> Callable[[str | None], TextIOWrapper]:
+    def _f(s: str | None) -> TextIOWrapper:
+        if s is None:
+            raise ValueError("No file specified.")
+        elif s == "-":
+            buffer = sys.stdin.buffer if mode.startswith("r") else sys.stdout.buffer
+            return TextIOWrapper(buffer)
+        elif s.endswith(".gz"):
+            buffer = gzip.open(s, mode=mode)
+            return TextIOWrapper(buffer)  # type: ignore
+        else:
+            return open(s, mode=mode)  # type: ignore
+
+    return _f
+
+
+def _parse_col_list(s: str | None) -> list[str] | None:
+    if s is None:
+        return s
+    else:
+        return s.split(",")
+
+
 def handler(args: argparse.Namespace) -> int:
     in_file: TextIOWrapper = args.in_file
     has_header: bool = args.header
     delimiter: str = args.delimiter
+    pick: list[str] | None = args.pick
+    omit: list[str] | None = args.omit
     data = read_file(in_file, delimiter, has_header)
+    out_cols = _prepare_out_cols(data, pick, omit)
 
     popts = PlotOptions.from_namespace(args, data.header)
     copts = CropOptions.from_namespace(args, data.header)
@@ -133,7 +191,7 @@ def handler(args: argparse.Namespace) -> int:
 
     if copts.start is not None:
         cropped_data = crop(data, copts)
-        return write_records(cropped_data, out_file, delimiter)
+        return write_records(cropped_data, out_file, delimiter=delimiter, cols=out_cols)
     else:
         fig, ax = plot(data, popts)
         return write_figure(fig, ax, out_file)
@@ -182,6 +240,23 @@ def _determine_numeric_columns(records: list[list[Any]]) -> list[bool]:
     return numeric
 
 
+def _prepare_out_cols(
+    data: Data,
+    pick_cols: list[str] | None,
+    omit_cols: list[str] | None,
+) -> list[int] | None:
+    header = data.header or list(range(len(data.records[0])))
+    hmap = {h: i for i, h in enumerate(header)}
+
+    if pick_cols is not None:
+        return [int(c) if c.isdigit() else hmap[c] for c in pick_cols]
+    elif omit_cols is not None:
+        _omit_idx = {int(c) if c.isdigit() else hmap[c] for c in omit_cols}
+        return [i for i, _ in enumerate(header) if i not in _omit_idx]
+    else:
+        return [i for i, _ in enumerate(header)]
+
+
 class CropOptions(NamedTuple):
     col: int
     start: Any
@@ -189,9 +264,9 @@ class CropOptions(NamedTuple):
 
     @classmethod
     def from_namespace(cls, args: argparse.Namespace, header: list[str] | None = None):
-        col = _get_column_index(args.crop_col, header)
-        start = coerce(args.crop_start)
-        end = cls._parse_endpoint(start, args.crop_end)
+        col = _get_column_index(args.col, header)
+        start = coerce(args.start)
+        end = cls._parse_endpoint(start, args.end)
         return cls(col, start, end)
 
     @staticmethod
@@ -287,12 +362,17 @@ def _argmin(values: list[int | float]) -> int:
 def write_records(
     data: Data,
     out_file: TextIOWrapper,
+    /,
+    *,
     delimiter: str = Defaults.DELIMITER,
+    cols: list[int] | None = None,
 ) -> int:
     writer = csv.writer(out_file, delimiter=delimiter)
     if data.header is not None:
-        writer.writerow(data.header)
-    writer.writerows(data.records)
+        h = [data.header[i] for i in cols] if cols is not None else data.header
+        writer.writerow(h)
+    r = [[rec[i] for i in cols] for rec in data.records] if cols else data.records
+    writer.writerows(r)
 
     return 0
 
